from .util import *


class __THMLeaderboard(object):
    def get_leaderboard(self, monthly=True, country=None) -> list:
        """
        Returns the top 50 users from the all-time leaderboard

        :param monthly Switch for the monthly data
        :type monthly: bool
        :param country: Country code for country leaderboards
        :type country: str
        :return: List containing top 50 users
        """

        queries = []
        if monthly:
            queries.append('type=monthly')

        if country:
            queries.append(f'country={country}')

        query = '?' + '&'.join(queries)

        return http_get(self.session, '/api/leaderboards' + query)['ranks']



from .util import *


class __THMKoth(object):
    def koth_machine_pool(self) -> list:
        """
        Gets the current koth machine pool

        :return: Machine list
        """

        return http_get(self.session, '/games/koth/get/machine-pool')

    def koth_game_data(self, game_id) -> dict:
        """
        Gets data about a koth game

        :type game_id: int
        :param game_id: Game id
        :return: Game data
        """

        return http_get(self.session, f'/games/koth/data/{game_id}', has_success=True)

    def koth_recent_games(self) -> list:
        """
        Gets 5 most recently finished koth games

        :return: List of 4 most recent koth games
        """

        return http_get(self.session, '/games/koth/recent/games')



from .util import *


class __THMUser(object):
    def user_exists(self, username) -> bool:
        """
        Checks if a username is taken

        :param username: Username to check
        :return:
        """

        return http_get(self.session, f'/api/user-exist/{username}')['success']

    def user_created_rooms(self, username) -> list:
        """
        Gets a list of rooms created by a user

        :param username: Username to check
        :return: List of rooms created by this user
        """

        return http_get(self.session, f'/api/created-rooms/{username}', has_success=True)['rooms']

    def user_all_completed_rooms(self, username) -> list:
        """
        Gets all rooms completed by a user

        :param username: Username to check
        :return: List of rooms (with all data) completed by this user
        """

        return http_get(self.session, f'/api/all-completed-rooms/{username}')

    def user_badges(self, username) -> list:
        """
        Gets the list of badges a user has

        :param username: Username to check
        :return: List of badges
        """

        return http_get(self.session, f'/api/get-badges/{username}')

    def user_activity(self, username) -> list:
        """
        Gets the user's activity feed

        :param username: Username to check
        :return: List of events
        """

        return http_get(self.session, f'/api/get-activity-events/{username}', has_success=True)['data']

    def user_discord(self, token) -> dict:
        """
        Gets the user's data from his discord integration token

        :param token: Discord integration token
        :return: User data
        """

        return http_get(self.session, f'/tokens/discord/{token}', has_success=True)

    def user_stats(self, username) -> dict:
        """
        Gets the user's basic data

        :param username: Username to check
        :return: User data (rank, points, avatar)
        """

        return http_get(self.session, f'/api/user/{username}')

    def user_rank(self, username) -> int:
        """
        Gets the user's rank

        :param username: Username to check
        :return: User's rank
        """

        return http_get(self.session, f'/api/usersRank/{username}')

    def user_badge_info(self, *badges) -> list:
        """
        Returns a list of badge objects

        :param badges: Badge list
        :return:
        """

        return http_post(self.session, '/api/get-specific-badges', data={'badges': badges})



from .auth import __THMAuth
from .instance import __THMInstance
from .koth import __THMKoth
from .leaderboard import __THMLeaderboard
from .room import __THMRoom
from .stats import __THMStats
from .team import __THMTeam
from .user import __THMUser
import requests


class THM(
    __THMAuth,
    __THMStats,
    __THMLeaderboard,
    __THMTeam,
    __THMUser,
    __THMKoth,
    __THMRoom,
    __THMInstance
):
    """
    TryHackMe API Wrapper
    """

    def __init__(self, credentials=None):
        """
        Initializes the API Wrapper

        :type credentials: dict
        :param credentials: (Optional) Credentials for use with authenticated requests
        """

        self.session = requests.Session()
        self.authenticated = False

        if (credentials is not None) and (type(credentials) == dict):
            if ('username' in credentials) and ('password' in credentials) or 'session' in credentials:
                self.login(credentials)



from .util import *
from requests.cookies import create_cookie


class __THMAuth(object):
    def login(self, credentials):
        """
        Log in to THM

        :param credentials: Login and password OR a session cookie
        :return: null
        """

        if 'session' not in credentials:
            csrf_token = fetch_pattern(self.session, '/login', 'csrf-input')

            data = {
                'email': credentials['username'],
                'password': credentials['password'],
                '_csrf': csrf_token
            }

            http_post(self.session, '/login', data, res_format='')
        elif 'session' in credentials:
            cookie = create_cookie('connect.sid', credentials['session'], domain='tryhackme.com')
            self.session.cookies.set_cookie(cookie)

        try:
            test_request = http_get(self.session, '/message/get-unseen')

            if test_request['success']:
                self.authenticated = True
                return
            else:
                raise Exception(f'Failed to authenticate')
        except Exception as e:
            raise e
